/****************************************************************************
 *
 *	  Header Filename:  ActReq_Requestor.cpp
 *	  Module Name:      Requestor for an Action_Request
 *	  Description:      Class used to hold parameters for creation of
 *                       an Action_Request, and to trigger its creation
 *
 *	  Author:           Chris Gordon-Smith
 *
 *     Copyright (C) 2008 Chris Gordon-Smith
 *     Email: See http://www.simsoup.info/
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License as published by the
 *     Free Software Foundation; either version 2 of the License, or (at your
 *     option) any later version.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc.,
 *     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *     See the file Licence.txt for a copy of the GNU General Public License.
 *	  Amendment History:-
 *
 *         Version    Date                 Update Description
 *         =======    ===========          ==================
 *
 *
 *
 *
 ****************************************************************************/

//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  


#include "globldef.h"
#include "Library_Headers.h"

#include "Kernel_Codes.h"

//--------------  Simulation  Subsystem  -----------------------------------------------------------------------

#include "ActReq_Requestor.h"
#include "Action_Request.h"
#include "Attribute.h"
//--------------------------------------------------------------------------------------------------------------

//--------------  Persistent Data Manager Subsystem ------------------------------------------------------------
#include "Simulation_Input.h"
#include "Input_Record.h"
//--------------------------------------------------------------------------------------------------------------

ActReq_Requestor::ActReq_Requestor()

{
    TRACE("ActReq_Requestor::ActReq_Requestor()");
}

//----------------------------------------------------------------------------------------------------------------
bool ActReq_Requestor::Get_Common_Attributes(Input_Record*  Line_Ptr, string& Error_Text)

//  Get the common attributes from the specified Input Line

{
    TRACE("ActReq_Requestor::Get_Common_Attributes");
    using namespace Attribute;
    
    string AttrName = AttrNames::Cd_To_Name(Time_Cd);
    
//  Action Time
    if (not Line_Ptr->Get_Attr_As_NonNegInt(AttrName, Action_Time_, Error_Text)) return false;
    
//  Repeat Specification
    AttrName = AttrNames::Cd_To_Name(Repeat_Spec_Cd);
    string Attr_Text;
    if (not Line_Ptr->Get_Attr_As_String(AttrName,
            Attr_Text,
            Error_Text)) return false;
    if (Attr_Text == "No")
    THEN
        Repeat_Interval_ = 0;
    ELSE
        if (not Text_To_PosInt(Attr_Text, Repeat_Interval_))
        THEN
            Error_Text = "Invalid Repeat Specification - " + Attr_Text + ": Expected the text \"No\" or a positive integer";
        return false;
        ENDIF
    ENDIF
    return true;
}
//----------------------------------------------------------------------------------------------------------------

bool  ActReq_Requestor::Proc_Rcd_Semantics(Input_Record*  Record_Ptr, string& Error_Text)

//  Process the semantics of an input record for an Action Request requestor
//  This is the base class routine, which performs common processing and invokes the
//  derived class routine to check the semantics for the particular kind of ActReq Requestor involved

{
    TRACE("ActReq_Requestor::Proc_Rcd_Semantics");
    bool Error_Fl = false;

    if (not Get_Common_Attributes(Record_Ptr, Error_Text))
    THEN
        Error_Fl = true;
    ELSE
        if (not this->Proc_Rcd_Semantics_Specific(Record_Ptr, Error_Text))
        THEN
            Error_Fl = true;
        ELSE
            if (not Record_Ptr->Check_All_Attributes_Read(Error_Text))
            THEN
                Error_Text = "Unexpected Attribute " 
                            + String_In_Quotes(string("@") + Error_Text);
                Error_Fl = true;
            ENDIF
        ENDIF
    ENDIF

    if (Error_Fl)
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

Add_Constr_ReqRqstr::Add_Constr_ReqRqstr():

    Num_To_Add_(0),
    R1_Spec_Pt_(0),
    R2_Spec_Pt_(0),
    P_Spec_Pt_(0),
    ACE_Spec_Pt_(0)

{        
    TRACE("Add_Constr_ReqRqstr::Add_Constr_ReqRqstr"); 
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Add_Constr_ReqRqstr::Make_ActReq()

//  Make an Action Request Requestor for a Construction

{
    TRACE("Add_Constr_ReqRqstr::Make_ActReq");    

    Add_Constr_Req*  Req_Ptr;
    CREATE_OBJECT(Add_Constr_Req, Req_Ptr =  new Add_Constr_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Add_Constr_ReqRqstr::~Add_Constr_ReqRqstr()

{
    TRACE("Add_Constr_ReqRqstr::~Add_Constr_ReqRqstr");
    DELETE_OBJECT(MolType_Spec, R1_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, R2_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec, ACE_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------

bool    Add_Constr_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rcd_Ptr, string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("Add_Constr_ReqRqstr::Proc_Rcd_Semantics_Specific");
    using namespace Attribute;
    
//  Setup Num_To_Add
    if (not Rcd_Ptr->Get_Attr_As_PosInt(
            AttrNames::Cd_To_Name(Num_To_Add_Cd),
            Num_To_Add_,
            Error_Text)) return false;

    bool  Rand_Flag;
    int   MolType_Id;

//  Setup the Reactant 1 MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Reactant1_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, R1_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Reactant 2 MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Reactant2_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, R2_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Product MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Product_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, P_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Activated Complex Energy (ACE)
    bool   Rnd_ACE_Fg;
    double ACE;  
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_Float(
            AttrNames::Cd_To_Name(ACE_Cd),
            Rnd_ACE_Fg,
            ACE,
            Error_Text)) return false;
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(Rnd_ACE_Fg, ACE));

    return true;    
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Add_Fission_ReqRqstr::Make_ActReq()

//  Make an Action Request Requestor for a Fission

{
    TRACE("Add_Fission_ReqRqstr::Make_ActReq");    

    Add_Fission_Req*  Req_Ptr;
    CREATE_OBJECT(Add_Fission_Req, Req_Ptr =  new Add_Fission_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Add_Fission_ReqRqstr::Add_Fission_ReqRqstr():

    Num_To_Add_(0),
    R_Spec_Pt_(0),
    P1_Spec_Pt_(0),
    P2_Spec_Pt_(0),
    ACE_Spec_Pt_(0)

{        
    TRACE("Add_Fission_ReqRqstr::Add_Fission_ReqRqstr"); 
}
//----------------------------------------------------------------------------------------------------------------

Add_Fission_ReqRqstr::~Add_Fission_ReqRqstr()

{
    TRACE("Add_Fission_ReqRqstr::~Add_Fission_ReqRqstr");
    DELETE_OBJECT(MolType_Spec, R_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P1_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P2_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec, ACE_Spec_Pt_);
}
//-----------------------------------------------------------------------------------------------------------------

bool    Add_Fission_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rcd_Ptr, string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("Add_Fission_ReqRqstr::Proc_Rcd_Semantics_Specific");
    using namespace Attribute;
    
//  Setup Num_To_Add
    if (not Rcd_Ptr->Get_Attr_As_PosInt(
            AttrNames::Cd_To_Name(Num_To_Add_Cd),
            Num_To_Add_,
            Error_Text)) return false;

    bool  Rand_Flag;
    int   MolType_Id;

//  Setup the Reactant MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Reactant_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, R_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Product 1 MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Product1_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, P1_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Product 2 MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Product2_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, P2_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Activated Complex Energy (ACE)
    bool   Rnd_ACE_Fg;
    double ACE;  
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_Float(
            AttrNames::Cd_To_Name(ACE_Cd),
            Rnd_ACE_Fg,
            ACE,
            Error_Text)) return false;
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(Rnd_ACE_Fg, ACE));

    return true;    
}
//-----------------------------------------------------------------------------------------------------------------

bool  ActReq_Requestor::Check_RecType_And_Make_Rqstr(Input_Record*  Rec_Ptr,
        ActReq_Requestor*&  Rqstr,
        string&  Error_Text)

//  Check Input Record Type and create an Act Req Requestor of the appropriate type
//  After this switch, handling of Action Request Requestors and Action Requests should
//  be dynamically polymorphic with no further switches required

{
    TRACE("ActReq_Requestor::Check_RecType_And_Make_Rqstr");
    SimInp::Record_Type_Code  Rcd_Type;
    string Rec_Tp_Text = Rec_Ptr->Get_Record_Type_Text();
    if (not SimInp::Get_Ptr()->Text_To_Record_Type(Rec_Tp_Text, Rcd_Type, Error_Text)) return false;
    switch (Rcd_Type)
            
    {
        case  SimInp::ChemEvol_Req_Rcd_Cd:
            CREATE_OBJECT(ChemEvol_ReqRqstr, Rqstr = new ChemEvol_ReqRqstr);
            break;
        case  SimInp::Add_Constr_Req_Rcd_Cd:
            CREATE_OBJECT(Add_Constr_ReqRqstr, Rqstr = new Add_Constr_ReqRqstr);
            break;

        case  SimInp::Add_Fission_Req_Rcd_Cd:
            CREATE_OBJECT(Add_Fission_ReqRqstr, Rqstr = new Add_Fission_ReqRqstr);
            break;

        case  SimInp::Add_Transfm_Req_Rcd_Cd:
            CREATE_OBJECT(Add_Transfm_ReqRqstr, Rqstr = new Add_Transfm_ReqRqstr);
            break;

        case  SimInp::Add_Mol_Req_Rcd_Cd:
            CREATE_OBJECT(Add_Molecule_ReqRqstr, Rqstr = new Add_Molecule_ReqRqstr);
            break;

        case  SimInp::Add_MolType_Req_Rcd_Cd:
            CREATE_OBJECT(Add_MolType_ReqRqstr, Rqstr = new Add_MolType_ReqRqstr);
            break;

        default:
            Assert(false, "ActReq_Requestor::Check_RecType_And_Make_Rqstr - Invalid Record Type following earlier validation");
            return false;
    }
    return true;
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Add_Molecule_ReqRqstr::Make_ActReq()

//  Make Action Request based on this Requestor

{
    TRACE("Add_Molecule_ReqRqstr::Make_ActReq");

    Add_Molecule_Req*  Req_Ptr;
    CREATE_OBJECT(Add_Molecule_Req, Req_Ptr =  new Add_Molecule_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Add_Molecule_ReqRqstr::Add_Molecule_ReqRqstr():

    MolType_Spec_Pt_ (0)
    
{
    TRACE("Add_Molecule_ReqRqstr()::Add_Molecule_ReqRqstr()");
}
//----------------------------------------------------------------------------------------------------------------

Add_Molecule_ReqRqstr::~Add_Molecule_ReqRqstr()

{
    TRACE("Add_Molecule_ReqRqstr()::~Add_Molecule_ReqRqstr()");
    DELETE_OBJECT(MolType_Spec, MolType_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------


bool   Add_Molecule_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rec_Ptr, 
                                                          string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("Add_Molecule_ReqRqstr::Proc_Rcd_Semantics_Specific");
    using namespace Attribute;
    
//  Setup Num_To_Add
    if (not Rec_Ptr->Get_Attr_As_PosInt(
            AttrNames::Cd_To_Name(Num_To_Add_Cd),
            Num_To_Add_,
            Error_Text)) return false;
//  Setup the MolType_Spec
    bool  Rand_Flag;
    int   MolType_Id;
    if (not Rec_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(MolType_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, MolType_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

    return true;
}
//----------------------------------------------------------------------------------------------------------------

Add_MolType_ReqRqstr::Add_MolType_ReqRqstr():

    Mass_Spec_Pt_ (0),  
    PotEnergy_Spec_Pt_ (0)
{
    TRACE("Add_MolType_ReqRqstr::Add_MolType_ReqRqstr()");
}
//----------------------------------------------------------------------------------------------------------------

Add_MolType_ReqRqstr::~Add_MolType_ReqRqstr()

{
    TRACE("Add_MolType_ReqRqstr::~Add_MolType_ReqRqstr()");
//    DELETE_OBJECT(Mass_Spec, Mass_Spec_Pt_);
//    DELETE_OBJECT(PotEnergy_Spec, PotEnergy_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Add_MolType_ReqRqstr::Make_ActReq()

//  Make an Add_MolType_Req Action Request based on this Requestor

{
    TRACE("Add_MolType_ReqRqstr::Make_ActReq");

    Add_MolType_Req*  Req_Ptr;
    CREATE_OBJECT(Add_MolType_Req, Req_Ptr =  new Add_MolType_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

bool   Add_MolType_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rec_Ptr,
                                                         string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("Add_MolType_ReqRqstr::Proc_Rcd_Semantics_Specific");
    using namespace Attribute;
    
//  Setup Num_To_Add
    if (not Rec_Ptr->Get_Attr_As_PosInt(
            AttrNames::Cd_To_Name(Num_To_Add_Cd),
            Num_To_Add_,
            Error_Text)) return false;

//  Setup the Mass_Spec
    bool  Rand_Mass_Flag;
    int   Mol_Mass;
    if (not Rec_Ptr->Get_Attr_As_Rand_Or_PosInt(
            AttrNames::Cd_To_Name(Mass_Cd),
            Rand_Mass_Flag,
            Mol_Mass,
            Error_Text)) return false;
    CREATE_OBJECT(Mass_Spec, Mass_Spec_Pt_ = new Mass_Spec(Rand_Mass_Flag, Mol_Mass));
    
//  Setup Pot_Energy
    bool   Rand_PE_Flag;
    double PotEnergy;
    if (not Rec_Ptr->Get_Attr_As_Rand_Or_Float(
            AttrNames::Cd_To_Name(Pot_Energy_Cd),
            Rand_PE_Flag,
            PotEnergy,
            Error_Text)) return false;
    CREATE_OBJECT(PotEnergy_Spec, PotEnergy_Spec_Pt_ = new PotEnergy_Spec(Rand_PE_Flag, PotEnergy));
    
    return true;
}
//----------------------------------------------------------------------------------------------------------------

Add_Transfm_ReqRqstr::Add_Transfm_ReqRqstr():

    Num_To_Add_(0),
    R_Spec_Pt_(0),
    P_Spec_Pt_(0),
    ACE_Spec_Pt_(0)

{        
    TRACE("Add_Transfm_ReqRqstr::Add_Transfm_ReqRqstr"); 
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Add_Transfm_ReqRqstr::Make_ActReq()

//  Make an Action Request Requestor for a Transformation

{
    TRACE("Add_Transfm_ReqRqstr::Make_ActReq");    

    Add_Transformation_Req*  Req_Ptr;
    CREATE_OBJECT(Add_Transformation_Req, Req_Ptr =  new Add_Transformation_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Add_Transfm_ReqRqstr::~Add_Transfm_ReqRqstr()

{
    TRACE("Add_Transfm_ReqRqstr::~Add_Transfm_ReqRqstr");
    DELETE_OBJECT(MolType_Spec, R_Spec_Pt_);
    DELETE_OBJECT(MolType_Spec, P_Spec_Pt_);
    DELETE_OBJECT(ACE_Spec, ACE_Spec_Pt_);
}
//----------------------------------------------------------------------------------------------------------------

bool    Add_Transfm_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rcd_Ptr, 
                                                          string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("Add_Transfm_ReqRqstr::Proc_Rcd_Semantics_Specific");
    using namespace Attribute;
    
//  Setup Num_To_Add
    if (not Rcd_Ptr->Get_Attr_As_PosInt(
            AttrNames::Cd_To_Name(Num_To_Add_Cd),
            Num_To_Add_,
            Error_Text)) return false;

    bool  Rand_Flag;
    int   MolType_Id;

//  Setup the Reactant MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Reactant_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, R_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Product MolType_Spec
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_NonNegInt(
            AttrNames::Cd_To_Name(Product_Cd),
            Rand_Flag,
            MolType_Id,
            Error_Text)) return false;
    CREATE_OBJECT (MolType_Spec, P_Spec_Pt_ = new MolType_Spec(Rand_Flag, MolType_Id))

//  Setup the Activated Complex Energy (ACE)
    bool   Rnd_ACE_Fl;
    double ACE;  
    if (not Rcd_Ptr->Get_Attr_As_Rand_Or_Float(
            AttrNames::Cd_To_Name(ACE_Cd),
            Rnd_ACE_Fl,
            ACE,
            Error_Text)) return false;
    CREATE_OBJECT(ACE_Spec, ACE_Spec_Pt_ = new ACE_Spec(Rnd_ACE_Fl, ACE));

    return true;    
}
//----------------------------------------------------------------------------------------------------------------


ChemEvol_ReqRqstr::ChemEvol_ReqRqstr()

{
    TRACE("ChemEvol_ReqRqstr::ChemEvol_ReqRqstr()");
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  ChemEvol_ReqRqstr::Make_ActReq()

//  Make an Action Request Requestor for a Chemistry Evolution step

{
    TRACE("ChemEvol_ReqRqstr::Make_ActReq");    

    ChemEvol_Req*  Req_Ptr;
    CREATE_OBJECT(ChemEvol_Req, Req_Ptr =  new ChemEvol_Req(this));
    return Req_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

ChemEvol_ReqRqstr::~ChemEvol_ReqRqstr()

{
    TRACE("ChemEvol_ReqRqstr::~ChemEvol_ReqRqstr");
}
//----------------------------------------------------------------------------------------------------------------

bool    ChemEvol_ReqRqstr::Proc_Rcd_Semantics_Specific(Input_Record*  Rcd_Ptr, string& Error_Text)

//  Check correctness of the non-common (specific) attributes from the
//  specified input record and setup the Requestor accordingly

{
    TRACE("ChemEvol_ReqRqstr::Proc_Rcd_Semantics_Specific");

//  No non-common attributes for this input record, so nothing to do

    return true;    
}
//----------------------------------------------------------------------------------------------------------------













/*
 * Action_Request* Add_Cat_Trans_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Cat_Trans_Req, Add_Cat_Trans_Req* Ptr = new Add_Cat_Trans_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_Construction_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Construction_Req, Add_Construction_Req* Ptr = new Add_Construction_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_Fission_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Fission_Req, Add_Fission_Req*  Ptr = new Add_Fission_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_Maint_MolType_Spec_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Maint_MolType_Spec_Req, Add_Maint_MolType_Spec_Req*
 * Ptr = new Add_Maint_MolType_Spec_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_MolType_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_MolType_Req, Add_MolType_Req*  Ptr = new Add_MolType_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_Molecule_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Molecule_Req, Add_Molecule_Req*  Ptr = new Add_Molecule_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Add_Transformation_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Add_Transformation_Req, Add_Transformation_Req*  Ptr = new Add_Transformation_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Empty_Reactor_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Empty_Reactor_Req, Empty_Reactor_Req*  Ptr = new Empty_Reactor_Req(this));
 * return Ptr;
 * }
 *
 * //Action_Request* Evolution_Step_ReqRqstr::Make_ActReq()
 * //{
 * //  	CREATE_OBJECT(Evolution_Step_Req, Evolution_Step_Req*  Ptr = new Evolution_Step_Req(this));
 * //  	return Ptr;
 * //}
 *
 * Action_Request* Remove_Fission_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Remove_Fission_Req, Remove_Fission_Req*  Ptr = new Remove_Fission_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Remove_Construction_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Remove_Construction_Req, Remove_Construction_Req*
 * Ptr = new Remove_Construction_Req(this));
 * return Ptr;
 * }
 *
 *
 * Action_Request* Remove_Maint_MolType_Spec_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Remove_Maint_MolType_Spec_Req, Remove_Maint_MolType_Spec_Req*
 * Ptr = new Remove_Maint_MolType_Spec_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Remove_Transformation_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Remove_Transformation_Req, Remove_Transformation_Req*
 * Ptr = new Remove_Transformation_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Set_Reactor_Vol_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Set_Reactor_Vol_Req, Set_Reactor_Vol_Req*
 * Ptr = new Set_Reactor_Vol_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Set_Max_Rand_Rate_Const_Constr_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Set_Max_Rand_Rate_Const_Constr_Req, Set_Max_Rand_Rate_Const_Constr_Req*
 * Ptr = new Set_Max_Rand_Rate_Const_Constr_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Set_Max_Rand_Rate_Const_Fission_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Set_Max_Rand_Rate_Const_Fission_Req, Set_Max_Rand_Rate_Const_Fission_Req*
 * Ptr = new Set_Max_Rand_Rate_Const_Fission_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Set_Max_Rate_Const_Transformation_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Set_Max_Rate_Const_Transformation_Req, Set_Max_Rate_Const_Transformation_Req*
 * Ptr = new Set_Max_Rate_Const_Transformation_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Set_Rate_Const_Model_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Set_Rate_Const_Model_Req, Set_Rate_Const_Model_Req*
 * Ptr = new Set_Rate_Const_Model_Req(this));
 * return Ptr;
 * }
 *
 * Action_Request* Terminate_Scenario_ReqRqstr::Make_ActReq()
 * {
 * CREATE_OBJECT(Terminate_Scenario_Req, Terminate_Scenario_Req*
 * Ptr = new Terminate_Scenario_Req(this));
 * return Ptr;
 * }
 *
 */
